GLOBAL.setmetatable(GLOBAL.getfenv(1), {__index = function(self, index)
	return GLOBAL.rawget(GLOBAL, index)
end})

local function overwrite(tabula, name, ante, post)
	if type(tabula) ~= "table" then return end 
	
	local unpack = unpack or table.unpack or GLOBAL.unpack
	
	local function results(data, ...)
		return type(data) == "function" and {data(...)}  
			or type(data) == "table" and data 
			or {data} 
	end 
	
	local old = tabula[name]
	tabula[name] = function(...)
		local results_ante = results(ante, ...)
		if #results_ante > 0 then return unpack(results_ante) end 
		
		local results_original = results(old, ...)
		
		local results_post = results(post, ...)
		if #results_post > 0 then return unpack(results_post) end 
		
		return unpack(results_original)
	end 
end 

local hacker = {}-- by the wonderful Rafael Lizarralde

local function GetUpvalueHelper(fn, name)
	local i = 1
	while debug.getupvalue(fn, i) and debug.getupvalue(fn, i) ~= name do
		i = i + 1
	end
	local name, value = debug.getupvalue(fn, i)
	return value, i
end

function hacker.get(fn, ...)
	local prv, i, prv_var = nil, nil, "(the starting point)"
	for j,var in ipairs({...}) do
		assert(type(fn) == "function", "We were looking for "..var..", but the value before it, "
			..prv_var..", wasn't a function (it was a "..type(fn)
			.."). Here's the full chain: "..table.concat({"(the starting point)", ...}, ", "))
		prv = fn
		prv_var = var
		fn, i = GetUpvalueHelper(fn, var)
	end
	return fn, i, prv
end

function hacker.set(start_fn, new_fn, ...)
	local _fn, _fn_i, scope_fn = hacker.get(start_fn, ...)
	debug.setupvalue(scope_fn, _fn_i, new_fn)
end

--			*			*			*			*			*			*			--

TUNING.PUMPKIN_LANTERN_RADIUS = 8 

local function flicker_update(inst)
    local time = GetTime()*30
	local flicker = ( math.sin( time ) + math.sin( time + 2 ) + math.sin( time + 0.7777 ) ) / 2.0 -- range = [-1 , 1]
	flicker = ( 1.0 + flicker ) / 2.0 -- range = 0:1
    flicker = flicker / 1.5 * TUNING.PUMPKIN_LANTERN_RADIUS-- the only line i added 
    inst.Light:SetRadius( 1.5 + 0.1 * flicker)
    inst.flickertask = inst:DoTaskInTime(0.1, function() flicker_update(inst) end)
end

local function ondropped(inst)
	local x, y, z = inst.Transform:GetWorldPosition()
	local tile = GetWorld().Map:GetTileAtPoint(x, y, z)
	if tile == GROUND.GASJUNGLE then 
		inst.components.perishable:Perish()
	end 
end

local function kill(inst)
	inst.components.health:SetInvincible(false)
	inst.components.health:Kill()
end

AddPrefabPostInit("pumpkin_lantern", function(inst)
	inst.Light:SetRadius(TUNING.PUMPKIN_LANTERN_RADIUS)
	inst.components.perishable.StartPerishing = function() end 
	inst:AddTag"notaget"
	inst:RemoveComponent"combat"
	if inst.components.health then inst.components.health:SetInvincible(true) end 
	inst:ListenForEvent("ondropped", ondropped)
	
	inst:AddComponent"workable"
	inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
	inst.components.workable:SetWorkLeft(1)
	inst.components.workable:SetOnFinishCallback(kill)
end)

AddPrefabPostInit("world", function(world)
	hacker.set(Prefabs.pumpkin_lantern.fn, flicker_update, "fade_in", "flicker_update")
end)